#!/usr/bin/env python3
from art import text2art
from shodanx.modules.logger.logger import Logger
import random

class Banner:
    def __init__(self, tool_name: str = "RevoltSecurities"):
        self.tool_name = tool_name
        self.fonts = ["big", "ogre", "shadow", "script", "graffiti", "slant"]
        self.logger = Logger(name="BannerPrinter")

    def render(self) -> str:
        selected_font = random.choice(self.fonts)
        banner_art = text2art(self.tool_name, font=selected_font)
        banner = f"""{self.logger.bold}{self.logger.random_color}{banner_art}{self.logger.reset}
                     {self.logger.bold}{self.logger.white}- RevoltSecurities{self.logger.reset}\n"""
        self.logger.banner(banner)


import asyncio
import click
import httpx
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanconfig.shodanconfig import ShodanxConfig
from shodanx.modules.help.help import Helper
from shodanx.modules.shodanauth.shodanauth import ShodanAuth
from shodanx.modules.shodancvedb.shodancvedb import ShodanxCVEDB
from shodanx.modules.shodanentity.shodanentity import ShodanxEntityDB
from shodanx.modules.shodanexposure.shodanexposure import ShodanExposureDB
from shodanx.modules.shodanfaviconmap.shodanfaviconmap import ShodanFaviconMap
from shodanx.modules.shodaninternetdb.shodaninternetdb import ShodanxInternetDB
from shodanx.modules.shodanmaps.shodanmaps import ShodanxMapsOSINT
from shodanx.modules.shodansearch.shodansearch import ShodanXWebSearch
from shodanx.modules.shodansubdomain.shodansubdomain import ShodanXSubdomainEnumerator
from shodanx.modules.shodantrends.shodantrends import ShodanxTrendsDB
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
from shodanx.modules.shodanxsave.shodanxsave import ShodanxSave
from shodanx.modules.gitutils.gitutils import GitUtils
from shodanx.modules.version.version import Version
from shodanx.modules.banner.banner import Banner
import tempfile
import os
import sys


class ShodanX:
    def __init__(self):
        self.logger = Logger()
        self.utils = ShodanxUtils()
        self.banner = Banner("shodanX")
        self.helps = Helper()
        self.config = ShodanxConfig()
        self.tmpdir = tempfile.gettempdir()
        self.saver = ShodanxSave(self.logger)
        self.git = GitUtils("RevoltSecurities/ShodanX", "shodanx", self.tmpdir)
        self.auth_yaml = self.config.config_auth()
        self.auth_cookie = self.config.config_cookie()
        self.version = Version()
        self.git_version = self.version.version
        self.pypi = self.version.pypi
        self.banner.render()
        self.Version()
    
    def show_help(self):
        self.banner.render()
        self.helps.main_help()
        
    def Version(self):
        async def versions():
            currentgit = await self.git.git_version()
            if not currentgit:
                self.logger.warn("unable to get the latest version of shodanx")
                return
            
            if currentgit == self.git_version:
                print(f"[{self.logger.blue}{self.logger.bold}version{self.logger.reset}]:{self.logger.bold}{self.logger.white}shodanx current version {self.git_version} ({self.logger.green}latest{self.logger.reset}{self.logger.bold}{self.logger.white}){self.logger.reset}", file=sys.stderr)
            else:
                print(f"[{self.logger.blue}{self.logger.bold}version{self.logger.reset}]:{self.logger.bold}{self.logger.white}shodanx current version {self.git_version} ({self.logger.red}outdated{self.logger.reset}{self.logger.bold}{self.logger.white}){self.logger.reset}", file=sys.stderr)
            return
        self._run_async(versions())

    def _run_async(self, coro):
        asyncio.run(coro)
        
    def auth(self,help):
        if help:
            self.helps.help_auth()
            return
        
        async def asyncauth():
            authenticator = ShodanAuth(self.auth_yaml, self.auth_cookie)
            data = await authenticator.shell()
            if data is None:
                self.logger.warn("No data found, please run the auth mode with proper and enter proper credentials")
                return
            setter = await self.utils.set_auth(self.auth_yaml, data.get("username"), data.get("password"), data.get("apikey"))
        self._run_async(asyncauth())
        
    def login(self,help):
        if help:
            self.helps.help_login()
            return

        async def asynclogin():
            async with httpx.AsyncClient(verify=False) as session:
                authenticator = ShodanAuth(self.auth_yaml, self.auth_cookie, session)
                cookies = await authenticator.shell_login()
                if cookies is None:
                    return
                setcookies = await self.utils.set_cookie(self.auth_cookie, cookies)
                self.logger.info("Logging into shodan with configured cookies, please wait :)")
                session = await authenticator.valid_session()
                if session:
                    self.logger.info("Successfully logged into shodan using shodanx authenticator")
                else:
                    self.logger.warn(f"Unable to log into shodan, please check your credenitals or manually set cookies in this file: {self.auth_cookie}")
                return
        self._run_async(asynclogin())
        
    def org(self, help,organization, facet, output, verbose):
        if help:
            self.helps.help_org()
            return
        
        if not organization:
            self.logger.warn("org search mode of shodanx requires organization name to pass!")
            return

        async def asyncorg():
            async with httpx.AsyncClient(verify=False) as session:
                shodansearch = ShodanXWebSearch(query=f'org:"{organization}"', facet=facet, session=session, verbose=verbose)
                response = await shodansearch.search()
                data = response.get("shodan_web_search", None)
                if data is None:
                    self.logger.warn(f"No results found for {organization}")
                    return
                for result in data:
                    self.logger.stdin(result)
                    if output:
                        await self.saver.save(output,result)

        self._run_async(asyncorg())

    def domain(self, help,domain, facet, output):
        if help:
            self.helps.help_domain()
            return
        
        if not domain:
            self.logger.warn("domain search mode requires domain name")
            return

        async def asyncdomain():
            async with httpx.AsyncClient(verify=False) as session:
                shodansearch = ShodanXWebSearch(query=f'hostname:"{domain}"', facet=facet, session=session)
                response = await shodansearch.search()
                data = response.get("shodan_web_search", None)
                if data is None:
                    self.logger.warn(f"No results found for {domain}")
                    return
                for result in data:
                    self.logger.stdin(result)
                    if output:
                        await self.saver.save(output,result)
                    
        self._run_async(asyncdomain())

    def ssl(self,help,query, facet, output):
        if help:
            self.helps.help_ssl()
            return
        
        if not query:
            self.logger.warn("ssl search mode requires a query")
            return

        async def asyncssl():
            async with httpx.AsyncClient(verify=False) as session:
                shodansearch = ShodanXWebSearch(query=query, facet=facet, session=session)
                response = await shodansearch.search()
                data = response.get("shodan_web_search", None)
                if data is None:
                    self.logger.warn(f"No results found for {query}")
                    return
                for result in data:
                    self.logger.stdin(result)
                    if output:
                        await self.saver.save(output,result)

        self._run_async(asyncssl())

    def custom(self, help,query, facet, output):
        if help:
            self.helps.help_custom()
            return
        
        if not query:
            self.logger.warn("custom search mode requires query")
            return

        async def asynccustom():
            async with httpx.AsyncClient(verify=False) as session:
                shodansearch = ShodanXWebSearch(query=query, facet=facet, session=session)
                response = await shodansearch.search()
                data = response.get("shodan_web_search", None)
                if data is None:
                    self.logger.warn(f"No results found for {query}")
                    return
                for result in data:
                    self.logger.stdin(result)
                    if output:
                        await self.saver.save(output,result)

        self._run_async(asynccustom())

    def subdomain(self,help, domain, output, verbose):
        if help:
            self.helps.help_subdomain()
            return
        if not domain:
            self.logger.warn("subdomain search requires domain")
            return

        async def asyncsubdomain():
            async with httpx.AsyncClient(verify=False) as session:
                shodanenum = ShodanXSubdomainEnumerator(domain, verbose=verbose, session=session)
                response = await shodanenum.enumerate()
                data = response.get("shodan_subdomains")
                if data is None:
                    self.logger.warn(f"No subdomains found for {domain}")
                    return
                for result in data:
                    self.logger.stdin(result)
                    if output:
                        await self.saver.save(output,result)

        self._run_async(asyncsubdomain())

    def cvedb(self, help,cpe, product, cve, exploited, start_date, end_date, limits, output):
        if help:
            self.helps.help_cvedb()
            return
        
        async def asynccve():
            async with httpx.AsyncClient(verify=False, follow_redirects=True) as session:
                cvemap = ShodanxCVEDB(session=session, is_kev=exploited, start_date=start_date, end_date=end_date, limit=limits)
                if cpe:
                    data = await cvemap.search(cpe=cpe)
                elif product:
                    data = await cvemap.search(product=product)
                elif cve:
                    data = await cvemap.search_cve_by_id(cve)
                else:
                    data = await cvemap.search()
                    
                if output:
                    await self.saver.save(output,data,True)
                    
        self._run_async(asynccve())

    def entitydb(self,help, id=None):
        if help:
            self.helps.help_entitydb()
            return
        
        async def asyncentitydb():
            async with httpx.AsyncClient(verify=False) as session:
                entitydbs = ShodanxEntityDB(session=session)
                data = await (entitydbs.list_entity_by_id(id) if id else entitydbs.list_all_entities())

        self._run_async(asyncentitydb())

    def exposuredb(self,help, country):
        if help:
            self.helps.help_exposuredb()
            return
        
        if not country:
            self.logger.warn("exposuredb requires country to search")
            return

        async def asyncexposure():
            async with httpx.AsyncClient(verify=False) as session:
                exposures = ShodanExposureDB(country=country, session=session)
                data = await exposures.search_exposure()

        self._run_async(asyncexposure())

    def faviconmap(self, help,top, all):
        if help:
            self.helps.help_faviconmap()
            return
        
        async def asyncfavicon():
            async with httpx.AsyncClient(verify=False) as session:
                favisearch = ShodanFaviconMap(all=all, top=top, session=session)
                data = await favisearch.search()

        self._run_async(asyncfavicon())

    def internetdb(self, help,domain, output, ips):
        if help:
            self.helps.help_internetdb()
            return
        
        async def asyncinternetdb():
            async with httpx.AsyncClient(verify=False) as session:
                dbsearch = ShodanxInternetDB(domain=domain, session=session, output=output)
                if domain:
                    data = await dbsearch.do()
                elif ips:
                    if os.path.exists(ips):
                        Ips = await self.utils.async_reader(ips)
                    else:
                        Ips = ips.split(",")
                    data = await dbsearch.search_all(Ips)

        self._run_async(asyncinternetdb())
        
    def trends(self,help,query,facet,output):
        if help:
            self.helps.help_trends()
            return
        
        async def asynctrends():
            async with httpx.AsyncClient(verify=False) as session:
                trendssearch = ShodanxTrendsDB(self.auth_yaml, query, session,facet)
                data = await trendssearch.search_trends()
                if output:
                    await self.saver.save(output,data,True)
                return
        self._run_async(asynctrends())
        
    def map(self,help,place,output):
        if help:
            self.helps.help_map()
            return
        
        async def asyncmaps():
            async with httpx.AsyncClient(verify=False) as session:
                mapsearch = ShodanxMapsOSINT(session=session)
                data = await mapsearch.search(cookies_file=self.auth_cookie, place=place)
                if output:
                    await self.saver.save(output,data,True)
                return
        self._run_async(asyncmaps())
        
    async def show_updates(self):
        await self.git.show_update_log()
        return
    
    def update(self,help,update,show_updates):
        if help:
            self.helps.help_update()
            return
        
        async def updatehandler():
            if show_updates:
                await self.show_updates()
                return
            
            if update:
                current = await self.git.git_version()
                if not current:
                    self.logger.warn("unable to get the latest version of shodanx")
                    return
                
                if current == self.git_version:
                    self.logger.info("ShodanX is already in latest version")
                    return
                
                zipurl = await self.git.fetch_latest_zip_url()
                if not zipurl:
                    self.logger.warn("unable to get the latest source code of RAI")
                    return
                
                await self.git.download_and_install(zipurl)
                newpypi = self.git.current_version()
                if newpypi == self.pypi:
                    self.logger.warn("unable to update RAI to the latest version, please try manually")
                    return
                
                self.logger.info(f"ShodanX has been updated to version")
                await self.show_updates()
                return
        self._run_async(updatehandler())

settings = dict(help_option_names=['-h', '--help'])

def customizer(ctx, param, value): 
    
    if value and not ctx.resilient_parsing:
        if not ctx.invoked_subcommand:
            Banner(tool_name="ShodanX").render()
            Helper().main_help()
            exit()
        else:
            ctx.invoke(ctx.command, ['--help'])

@click.group(context_settings=settings)
@click.option("-h", "--help", is_flag=True, is_eager=True, expose_value=False, callback=customizer)
def cli():
    pass


@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("--organization", "-org", type=str)
@click.option("--facet", "-fct", default="ip")
@click.option("--output", "-o", type=str)
@click.option("--verbose", "-v", is_flag=True)
def org(**kwargs):
    ShodanX().org(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("--domain", "-domain", type=str)
@click.option("--facet", "-fct", default="ip")
@click.option("--output", "-o", type=str)
def domain(**kwargs):
    ShodanX().domain(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("--ssl-query", "-sq", type=str)
@click.option("--facet", "-fct", default="ip")
@click.option("--output", "-o", type=str)
def ssl(**kwargs):
    ShodanX().ssl(help=kwargs["help"],query=kwargs["ssl_query"], facet=kwargs["facet"], output=kwargs["output"])

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("--custom-query", "-cq", type=str)
@click.option("--facet", "-fct", default="ip")
@click.option("--output", "-o", type=str)
def custom(**kwargs):
    ShodanX().custom(help=kwargs["help"],query=kwargs["custom_query"], facet=kwargs["facet"], output=kwargs["output"])

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("--domain", "-d", type=str)
@click.option("--output", "-o", type=str)
@click.option("--verbose", "-v", is_flag=True)
def subdomain(**kwargs):
    ShodanX().subdomain(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("-cpe", "--cpe", type=str)
@click.option("-product", "--product", type=str)
@click.option("-cve", "--cve", type=str)
@click.option("-exploited", "--exploited", is_flag=True)
@click.option("-sd","--start-date",  type=str,default=None)
@click.option("-ed","--end-date", type=str, default=None)
@click.option("-l", "--limits", type=int, default=1000)
@click.option("-o", "--output", type=str,default=None)
def cvedb(**kwargs):
    ShodanX().cvedb(**kwargs)


@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("-id", "--id", type=str)
def entitydb(**kwargs):
    ShodanX().entitydb(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("-country", "--country", type=str)
def exposuredb(**kwargs):
    ShodanX().exposuredb(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("-top", "--top", type=int, default=500)
@click.option("-all", "--all", is_flag=True)
def faviconmap(**kwargs):
    ShodanX().faviconmap(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
@click.option("-d", "--domain", type=str)
@click.option("-o", "--output", type=str)
@click.option("-ips", "--ips", type=str)
def internetdb(**kwargs):
    ShodanX().internetdb(**kwargs)

@cli.command()
@click.option("-h", "--help",is_flag=True)
def auth(**kwargs):
    ShodanX().auth(**kwargs)
    
@cli.command()
@click.option("-h", "--help",is_flag=True)
def login(**kwargs):
    ShodanX().login(**kwargs)
    
@cli.command()
@click.option("-h", "--help", is_flag=True)
@click.option("-query", "--query", type=str, default=None)
@click.option("-fct", "--facet", type=str, default="ip")
@click.option("-o", "--output", type=str, default=None)
def trends(**kwargs):
    ShodanX().trends(**kwargs)
    
@cli.command()
@click.option("-h", "--help", is_flag=True)
@click.option("-place", "--place", type=str,default=None)
@click.option("-o", "--output", type=str,default=None)
def map(**kwargs):
    ShodanX().map(**kwargs)

@cli.command()
@click.option("-h", "--help", is_flag=True)
@click.option("-up", "--update", is_flag=True)
@click.option("-sup", "--show-updates", is_flag=True)
def update(**kwargs):
    ShodanX().update(**kwargs)
    
if __name__ == "__main__":
    cli()


import os
import aiofiles
import asyncio
import importlib.metadata as metadata
import httpx
from rich.console import Console
from rich.markdown import Markdown
from shodanx.modules.logger.logger import Logger


class GitUtils:
    def __init__(self, repo: str, package: str, config_path: str):
        self.repo = repo
        self.package = package
        self.config_path = config_path
        self.console = Console()
        self.logger = Logger()

    def current_version(self) -> str:
        try:
            return metadata.version(self.package)
        except metadata.PackageNotFoundError:
            self.logger.warn(f"Package {self.package} not found.")
            return "Unknown"
        
    async def git_version(self) -> str:
        try:
            async with httpx.AsyncClient(timeout=10) as client:
                response = await client.get(f"https://api.github.com/repos/{self.repo}/releases/latest")
                if response.status_code == 200:
                    return response.json().get("tag_name")
                else:
                    return None
        except Exception as e:
            return None

    async def fetch_latest_zip_url(self) -> str | None:
        url = f"https://api.github.com/repos/{self.repo}/releases/latest"
        try:
            async with httpx.AsyncClient(timeout=10) as client:
                response = await client.get(url)
                if response.status_code == 200:
                    return response.json().get("zipball_url")
                else:
                    self.logger.warn("Failed to fetch latest release info from GitHub.")
        except Exception as e:
            self.logger.error(f"Error fetching update URL: {e}")
        return None

    async def download_and_install(self, zip_url: str):
        filepath = os.path.join(self.config_path, f"{self.package}.zip")
        try:
            self.logger.info("Downloading latest version...")
            async with httpx.AsyncClient(timeout=20) as client:
                async with client.stream("GET", zip_url) as response:
                    if response.status_code == 200:
                        async with aiofiles.open(filepath, "wb") as f:
                            async for chunk in response.aiter_bytes():
                                await f.write(chunk)
                    else:
                        self.logger.warn("Failed to download the latest release.")
                        return

            process = await asyncio.create_subprocess_exec(
                "pipx", "install", filepath, "--force",
                stdout=asyncio.subprocess.DEVNULL,
                stderr=asyncio.subprocess.DEVNULL
            )
            await process.communicate()
            if process.returncode == 0:
                self.logger.success(f"{self.package} updated successfully.")
            else:
                self.logger.warn("Update failed. Try manual update.")
        except Exception as e:
            self.logger.error(f"Update failed: {e}")
        finally:
            if os.path.exists(filepath):
                os.remove(filepath)

    async def show_update_log(self):
        url = f"https://api.github.com/repos/{self.repo}/releases/latest"
        try:
            async with httpx.AsyncClient(timeout=20) as client:
                response = await client.get(url)
                if response.status_code == 200:
                    log = response.json().get("body", "")
                    self.console.print(Markdown(log))
                else:
                    self.logger.info(f"Could not fetch logs. Visit: https://github.com/{self.repo}")
        except Exception as e:
            self.logger.error(f"Error fetching changelog: {e}")


import aiodns
from typing import List, Optional

class ShodanXDNSResolver:
    def __init__(self, nameservers: Optional[List[str]] = None, timeout: int = 30):
        if nameservers:
            self.nameservers = nameservers
        else:
            self.nameservers = None
        self.resolver = aiodns.DNSResolver(nameservers=self.nameservers)
    async def resolve(self, domain: str) -> List[str]:
        try:
            result = await self.resolver.query(domain, 'A')
            return [r.host for r in result]
        except aiodns.error.DNSError:
            return []
        except Exception as e:
            return []


import httpx
from bs4 import BeautifulSoup,XMLParsedAsHTMLWarning, MarkupResemblesLocatorWarning
import warnings
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils

class ShodanXWebSearch():
    def __init__(self, query: str, facet:str,session: httpx.AsyncClient, verbose:bool = False) -> None:
        self.sesssion = session
        self.facet = facet
        self.verbose = verbose
        self.query = query
        self.results = []
        self._shodan_utils = ShodanxUtils()
        self._logger = Logger()
        self._shodan_url =f"https://www.shodan.io/search/facet?query={self.query}&facet={self.facet}"
        self.data = {"shodan_web_search": []}
        self.headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Connection": "keep-alive",
            "Referer": "https://account.shodan.io/?language=en",
            "Upgrade-Insecure-Requests": "1",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Priority": "u=0, i",
            "Pragma": "no-cache",
            "Cache-Control": "no-cache",
            "TE": "trailers"
        }
    
    async def search(self) -> dict[str,str]:
        try:
            self.headers["User-Agent"] = self._shodan_utils.random_useragent()
            response = await self.sesssion.request("GET", self._shodan_url, headers=self.headers, timeout=30)
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore", category=UserWarning)
                warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)
                warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)
                soup = BeautifulSoup(response.text, "lxml")
                results = soup.find_all('strong')
                for result in results:
                    self.results.append(result.get_text())
                self.data['shodan_web_search'] = self.results
        except Exception as e:
            if self.verbose:
                self._logger.warn(f"error occured in the shodan search module due to: {e}")
        finally:
            return self.data
    


import httpx
from bs4 import BeautifulSoup,XMLParsedAsHTMLWarning, MarkupResemblesLocatorWarning
import warnings
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils

class ShodanXSubdomainEnumerator():
    def __init__(self, domain:str, session:httpx.AsyncClient, verbose:bool =False):
        self.domain = domain
        self.session = session
        self._shodan_url = f"https://www.shodan.io/domain/{self.domain}"
        self.results = []
        self.verbose = verbose
        self.data = {"shodan_subdomains": self.results}
        self._logger = Logger()
        self._shodan_utils = ShodanxUtils
        self.headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Connection": "keep-alive",
            "Referer": "https://account.shodan.io/?language=en",
            "Upgrade-Insecure-Requests": "1",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Priority": "u=0, i",
            "Pragma": "no-cache",
            "Cache-Control": "no-cache",
            "TE": "trailers"
        }
        
    async def enumerate(self) -> dict[str,str]:
        try:
            self.headers["User-Agent"] = self._shodan_utils.random_useragent()
            response: httpx.Response = await self.session.request("GET", self._shodan_url, headers=self.headers, timeout=30)
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore", category=UserWarning)
                warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)
                warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)
                soup = BeautifulSoup(response.text, "lxml")
                ul = soup.find('ul', id='subdomains')
                if ul:
                    results = ul.findAll("li")
                    for result in results:
                        self.results.append(result.text.strip()+f".{self.domain}")
                self.data["shodan_subdomains"] = self.results
        except Exception as e:
            self._logger.warn(f"error occured in shodanx subdomain enumeration due to: {e}")
        finally:
            return self.data


from shodanx.modules.shodaninternetdb.shodanresolver import ShodanXDNSResolver
import httpx
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
from shodanx.modules.shodanxsave.shodanxsave import ShodanxSave
from rich.console import Console
from rich.panel import Panel
from typing import List, Dict, Optional
from rich.table import Table
from rich.text import Text
from rich.box import DOUBLE_EDGE


class ShodanxInternetDB():
    def __init__(self, session: httpx.AsyncClient, domain: str = None, output: str=None):
        self.domain = domain
        self.output = output
        self.session = session
        self.logger = Logger()
        self.console = Console()
        self.saver = ShodanxSave(self.logger)
        self._shodan_utils = ShodanxUtils()
        self._shodan_url = "https://internetdb.shodan.io"
        self.results = []
        
    async def search(self, ip: str) -> dict:
        try:
            url = f"{self._shodan_url}/{ip}"
            response: httpx.Response = await self.session.request("GET", url, timeout=30)
            if response.status_code != 200:
                return None
            data = response.json()
            return {
                'ip': data['ip'],
                'ports': data.get('ports', []),
                'cpes': data.get('cpes', []),
                'hostnames': data.get('hostnames', []),
                'tags': data.get('tags', []),
                'vulns': data.get('vulns', [])
            }
        except Exception as e:
            self.logger.warn(f"Error occurred in shodan single search InternetDB module due to: {e}")
            return {}
        
    async def search_all(self, ips: List[str]) -> List[Dict]:
        try:
            results = []
            for ip in ips:
                result = await self.search(ip)
                if result:
                    results.append(result)
                    if self.output:
                        await self.saver.save(self.output,result,True)
                    self._display_single_result(result)
            return results
        except Exception as e:
            self.logger.warn(f"Error occurred in the shodan InternetDB search module due to: {e}")
            return []
        
    async def do(self):
        try:
            shodanresolver = ShodanXDNSResolver()
            ips = await shodanresolver.resolve(self.domain)
            if ips is None:
                return []
            results = await self.search_all(ips)
            if self.output:
                await self.saver.save(self.output, results, True)
            return results
        except Exception as e:
            self.logger.warn(f"Error occurred in the internetdb search do module due to: {e}")
            return []
            
    def _create_port_table(self, result: dict) -> Table:
        table = Table(
            box=DOUBLE_EDGE,
            header_style="bold white",
            expand=True,
            show_lines=True
        )
        
        table.add_column("IP", style="bold cyan", no_wrap=True)
        table.add_column("Port", justify="center")
        table.add_column("Service", style="bold white")
        table.add_column("Service Name", style="bold white")
        table.add_column("Tags", style="bold white")
        
        ip = result['ip']
        hostnames = "\n".join(result['hostnames']) if result['hostnames'] else "N/A"
        
        for port in result['ports']:
            style, service_label = self._shodan_utils.get_port_style_and_service(port)
            service_name = self._shodan_utils.get_service_name(port)
            
            table.add_row(
                f"{ip}\n[bold yellow]{hostnames}[/]",
                Text(str(port), style=style),
                Text(service_label, style=style),
                service_name,
                ", ".join(result['tags']) or "None"
            )
            
        return table

    def _create_vulnerability_table(self, result: dict) -> Optional[Table]:
        if not result['vulns']:
            return None
            
        table = Table(
            box=DOUBLE_EDGE,
            expand=True,
            style="bold white",
            show_lines=True
        )
        
        table.add_column("Vulnerability ID", style="bold green")
        table.add_column("Exposed Ports", style="bold cyan")
        table.add_column("Related CPEs", overflow="fold", style="bold yellow")
        
        for vuln in result['vulns']:
            table.add_row(
                vuln,
                ", ".join(str(p) for p in result['ports']),
                "\n".join(result['cpes'][:3]) + ("..." if len(result['cpes']) > 3 else "")
            )
            
        return table

    def _create_host_summary(self, result: dict) -> Table:
        """Create host summary for a single result"""
        table = Table(
            box=DOUBLE_EDGE,
            header_style="bold blue",
            expand=True,
            style="bold white",
            show_lines=True,
        )
        
        table.add_column("IP", style="bold white")
        table.add_column("Hostnames", overflow="fold", style="bold magenta")
        table.add_column("Open Ports", style="bold blue")
        table.add_column("Unique Services", style="bold cyan")
        table.add_column("Vulnerabilities", style="bold green")
        
        services = {
            self._shodan_utils.get_service_name(port)
            for port in result['ports']
        }
        
        table.add_row(
            result['ip'],
            "\n".join(result['hostnames']) or "None",
            str(len(result['ports'])),
            str(len(services)),
            str(len(result['vulns'])) + " found" if result['vulns'] else "None"
        )
        return table

    def _display_single_result(self, result: dict):
        """Display all visualizations for a single IP result"""
        if not result:
            return
            
        self.console.print(Panel.fit(
            self._create_host_summary(result),
            title=f"[bold green]Target Summary for {result['ip']}[/]",
            border_style="green",
        ))
        
        self.console.print(Panel.fit(
            self._create_port_table(result),
            title=f"[bold blue]Port Analysis for {result['ip']}[/]",
            border_style="blue"
        ))
        
        vuln_table = self._create_vulnerability_table(result)
        if vuln_table:
            self.console.print(Panel.fit(
                vuln_table,
                title=f"[bold red]Vulnerabilities Found for {result['ip']}[/]",
                border_style="bold white"
            ))
        else:
            self.console.print(f"[bold red]No vulnerabilities found for {result['ip']}[/]")
        print("\n")

    def display_results(self, results: List[Dict] = None):
        display_results = results or self.results
        if not display_results:
            return
            
        self.console.print(Panel.fit(
            self._create_aggregate_host_summary(display_results),
            title="[bold green]Target Summary[/]",
            border_style="green",
        ))
        
        self.console.print(Panel.fit(
            self._create_aggregate_port_table(display_results),
            title="[bold blue]Port Analysis[/]",
            border_style="blue"
        ))
        
        vuln_table = self._create_aggregate_vulnerability_table(display_results)
        if vuln_table:
            self.console.print(Panel.fit(
                vuln_table,
                title="[bold green]Vulnerabilities Found[/bold green]",
                border_style="bold white"
            ))
        else:
            self.console.print("[bold red]No vulnerabilities found in InternetDB data[/bold red]")

    def _create_aggregate_host_summary(self, results: List[Dict]) -> Table:
        table = Table(
            box=DOUBLE_EDGE,
            header_style="bold blue",
            expand=True,
            style="bold white",
            show_lines=True,
        )
        
        table.add_column("IP", style="bold white")
        table.add_column("Hostnames", overflow="fold", style="bold magenta")
        table.add_column("Open Ports", style="bold blue")
        table.add_column("Unique Services", style="bold cyan")
        table.add_column("Vulnerabilities", style="bold green")
        
        for result in results:
            services = {
                self._shodan_utils.get_service_name(port)
                for port in result['ports']
            }
            
            table.add_row(
                result['ip'],
                "\n".join(result['hostnames']) or "None",
                str(len(result['ports'])),
                str(len(services)),
                str(len(result['vulns'])) + " found" if result['vulns'] else "None"
            )
        return table

    def _create_aggregate_port_table(self, results: List[Dict]) -> Table:
        table = Table(
            box=DOUBLE_EDGE,
            header_style="bold white",
            expand=True,
            show_lines=True
        )
        
        table.add_column("IP", style="bold cyan", no_wrap=True)
        table.add_column("Port", justify="center")
        table.add_column("Service", style="bold white")
        table.add_column("Service Name", style="bold white")
        table.add_column("Tags", style="bold white")
        
        for result in results:
            ip = result['ip']
            hostnames = "\n".join(result['hostnames']) if result['hostnames'] else "N/A"
            
            for port in result['ports']:
                style, service_label = self._shodan_utils.get_port_style_and_service(port)
                service_name = self._shodan_utils.get_service_name(port)
                
                table.add_row(
                    f"{ip}\n[bold yellow]{hostnames}[/]",
                    Text(str(port), style=style),
                    Text(service_label, style=style),
                    service_name,
                    ", ".join(result['tags']) or "None"
                )
                
        return table

    def _create_aggregate_vulnerability_table(self, results: List[Dict]) -> Optional[Table]:
        vuln_data = []
        for result in results:
            if result['vulns']:
                for vuln in result['vulns']:
                    vuln_data.append({
                        'ip': result['ip'],
                        'vulnerability': vuln,
                        'ports': ", ".join(str(p) for p in result['ports']),
                        'cpes': "\n".join(result['cpes'][:3]) + ("..." if len(result['cpes']) > 3 else "")
                    })
        
        if not vuln_data:
            return None
            
        table = Table(
            box=DOUBLE_EDGE,
            expand=True,
            style="bold white",
            show_lines=True
        )
        
        table.add_column("IP", style="bold white")
        table.add_column("Vulnerability ID", style="bold green")
        table.add_column("Exposed Ports", style="bold cyan")
        table.add_column("Related CPEs", overflow="fold", style="bold yellow")
        
        for vuln in vuln_data:
            table.add_row(
                vuln['ip'],
                vuln['vulnerability'],
                vuln['ports'],
                vuln['cpes']
            )
            
        return table


from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

class Helper:
    def __init__(self) -> None:
        self.console = Console()
        
    def main_help(self):
        self.console.print(Panel.fit(
            Text("SHODANX ⚡ HELP", justify="center", style="bold blue"),
            border_style="blue",
            padding=(1, 4)
        ))

        self.console.print("[bold blue]DESCRIPTION[/bold blue]")
        self.console.print(
            "[bold white]\nShodanX is a terminal-powered recon and OSINT tool built on top of the Shodan Services.\n"
            "It empowers ethical hackers and red teamers to identify exposed infrastructure,\n"
            "search CVEs, map attack surfaces, and run internet-wide queries in real time.\n[/bold white]"
        )

        self.console.print("[bold blue]MODES (shodanx <mode>)[/bold blue]\n")
        mode_table = Table(show_header=True, header_style="bold magenta", show_lines=True)
        mode_table.add_column("Mode", style="bold cyan", no_wrap=True)
        mode_table.add_column("Description", style="bold white")

        modes = [
            ("auth", "Configure and save your Shodan API key for authenticated access."),
            ("login", "Verify your ShodanX access level and API key validity."),
            ("org", "Perform organization-wide scans using Shodan facets."),
            ("domain", "Enumerate metadata, ports, and host exposure for a domain."),
            ("subdomain", "Discover passive subdomains using Shodan’s DNS data."),
            ("ssl", "Search by SSL fingerprint to track reused/expired certs."),
            ("custom", "Execute advanced queries using Shodan filters and dorks."),
            ("internetdb", "Lightweight analysis of IPs/domains via Shodan InternetDB API."),
            ("map", "find exposed services and ports using Shodan Map"),
            ("cvedb", "Find new and CVE infromations via Shodan’s CVE database."),
            ("entitydb", "Explore technologies/vendors via Shodan’s EntityDB."),
            ("exposuredb", "Discover globally exposed assets and leaks by country."),
            ("faviconmap", "Find systems using specific favicons via Shodan FaviconMap."),
            ("trends", "View historical exposure trends (Shodan Enterprise required)."),
        ]
        for mode, desc in modes:
            mode_table.add_row(mode, desc)
        self.console.print(mode_table)

        self.console.print("[bold blue]FLAGS[/bold blue]")
        flag_table = Table(show_header=True, header_style="bold magenta")
        flag_table.add_column("Flag", style="bold cyan", no_wrap=True)
        flag_table.add_column("Description", style="bold white")
        flag_table.add_row("-h, --help", "Show this help message and exit.")
        self.console.print(flag_table)

        self.console.print("[bold blue]USAGE[/bold blue]")
        self.console.print("[bold white]shodanx <mode> [options][/bold white]\n")

        self.console.print("[bold blue]DETAILS[/bold blue]")
        detail_table = Table(show_header=False, box=None)
        for mode, desc in modes:
            detail_table.add_row(f"[bold white]shodanx[/] [bold cyan]{mode}[/]", f"[bold white]→ {desc}[/]")
        self.console.print(detail_table)

    def _print_mode_help(self, title: str, description: str, flags: list):
        self.console.print(Panel.fit(
            Text(title, justify="center", style="bold blue"),
            border_style="blue",
            padding=(1, 4)
        ))
        self.console.print(f"[bold blue]DESCRIPTION[/bold blue]\n[bold white]{description}[/bold white]\n")

        self.console.print("[bold blue]FLAGS[/bold blue]")
        flag_table = Table(show_header=True, header_style="bold magenta")
        flag_table.add_column("Flag", style="bold cyan", no_wrap=True)
        flag_table.add_column("Description", style="bold white")
        for flag, desc in flags:
            flag_table.add_row(flag, desc)
        self.console.print(flag_table)

    def help_org(self):
        self._print_mode_help(
            "ORG MODE",
            "Perform organization-wide scans using Shodan facets.",
            [
                ("--organization, -org", "Name of the target organization to investigate."),
                ("--facet, -fct", "Group results by a facet like 'ip', 'asn', or 'country'. Default is 'ip'."),
                ("--output, -o", "Specify output file to save results in JSON format."),
                ("--verbose, -v", "Enable detailed logs during execution.")
            ]
        )

    def help_domain(self):
        self._print_mode_help(
            "DOMAIN MODE",
            "Enumerate metadata, ports, and host exposure for a domain.",
            [
                ("--domain, -domain", "Domain name to scan for public exposure."),
                ("--facet, -fct", "Group results by facet such as 'port' or 'org'. Default is 'ip'."),
                ("--output, -o", "Save output to a file in structured JSON.")
            ]
        )

    def help_ssl(self):
        self._print_mode_help(
            "SSL MODE",
            "Search by SSL fingerprint to track reused/expired certs.",
            [
                ("--ssl-query, -sq", "SSL hash or certificate fingerprint to search."),
                ("--facet, -fct", "Group SSL certs by facet (e.g. org, product, port)."),
                ("--output, -o", "Path to file where results will be saved.")
            ]
        )

    def help_custom(self):
        self._print_mode_help(
            "CUSTOM MODE",
            "Execute advanced queries using Shodan filters and dorks.",
            [
                ("--custom-query, -cq", "Custom query using Shodan filters like 'port:80 country:IN'."),
                ("--facet, -fct", "Group matched data using facets like 'asn', 'port', etc."),
                ("--output, -o", "Output file to store query results.")
            ]
        )

    def help_subdomain(self):
        self._print_mode_help(
            "SUBDOMAIN MODE",
            "Discover passive subdomains using Shodan’s DNS data.",
            [
                ("--domain, -d", "Domain to extract subdomains from (e.g. example.com)."),
                ("--output, -o", "Save discovered subdomains to file."),
                ("--verbose, -v", "Enable detailed logging and subdomain count.")
            ]
        )

    def help_cvedb(self):
        self._print_mode_help(
            "CVEDB MODE",
            "Identify vulnerable systems via Shodan’s CVE database.",
            [
                ("--cpe, -cpe", "Common Platform Enumeration (CPE) string to filter results."),
                ("--product, -product", "Target product name (e.g., apache, nginx)."),
                ("--cve, -cve", "Specific CVE ID to search affected systems."),
                ("--exploited, -exploited", "Only show systems confirmed as exploited."),
                ("--start-date, -sd", "Filter results starting from this date (YYYY-MM-DD)."),
                ("--end-date, -ed", "Filter results ending at this date (YYYY-MM-DD)."),
                ("--limits, -l", "Limit number of results. Default is 1000."),
                ("--output, -o", "Save the found CVE results in a give file (only support JSON format)"),
            ]
        )

    def help_entitydb(self):
        self._print_mode_help(
            "ENTITYDB MODE",
            "Explore technologies/vendors via Shodan’s EntityDB.",
            [
                ("--id, -id", "Vendor or technology name/ID to investigate.")
            ]
        )

    def help_exposuredb(self):
        self._print_mode_help(
            "EXPOSUREDB MODE",
            "Discover globally exposed assets and leaks by country.",
            [
                ("--country, -country", "country name to search from the shodan exposure database")
            ]
        )

    def help_faviconmap(self):
        self._print_mode_help(
            "FAVICONMAP MODE",
            "Find systems using specific favicons via Shodan FaviconMap.",
            [
                ("--top, -top", "Limit results to top N most common favicons (default: 500)."),
                ("--all, -all", "Return all favicon matches regardless of count.")
            ]
        )

    def help_internetdb(self):
        self._print_mode_help(
            "INTERNETDB MODE",
            "Lightweight analysis of IPs/domains via Shodan InternetDB API.",
            [
                ("--domain, -d", "Domain to gather InternetDB intelligence data."),
                ("--output, -o", "Save summary output to a file."),
                ("--ips, -ips", "Comma-separated list of IPs to analyze (e.g. 1.1.1.1,8.8.8.8).")
            ]
        )

    def help_auth(self):
        self._print_mode_help(
            "AUTH MODE",
            "Configure and save your Shodan API key.",
            []
        )

    def help_login(self):
        self._print_mode_help(
            "LOGIN MODE",
            "Verify your ShodanX access level and API key validity and session cookies validity",
            []
        )

    def help_trends(self):
        self._print_mode_help(
            "TRENDS MODE",
            "View historical exposure trends (Shodan Enterprise API Key required).",
            [
                ("--query, -query", "Search term for trend analysis."),
                ("--facet, -fct", "Facet used for historical grouping (e.g. org, port)."),
                ("--output, -o", "Save output trends to a file."),
                ("--help, -h", "Display this help message for the trends mode.")
            ]
        )

    def help_map(self):
        self._print_mode_help(
            "MAP MODE",
            "Visualize geo-distribution of exposed services and ports.",
            [
                ("--place, -place", "Geographical target area (e.g. city, country, continent)."),
                ("--output, -o", "Path to output image or file."),
                ("--help, -h", "Display this help message for the map mode.")
            ]
        )

    def help_update(self):
        
        self._print_mode_help(
            "UPDATE MODE",
            "Handles the Updates & Updates information of ShodanX",
            [
                ("--show-updates, -sup", "shows the latest version updates information"),
                ("--update, -up", "updates the shodanx to latest version"),
                ("--help, -h", "Display this help message for the map mode.")
            ]
        )


from pathlib import Path
from appdirs import user_config_dir

class ShodanxConfig:
    def __init__(self, app_name: str = "ShodanX"):
        self.app_name = app_name
        if not self.app_name:
            raise ValueError("App name is required")
        self._ensure_config_dir_exists()
        self._ensure_config_files_exist()
        
    @property
    def config_dir(self) -> Path:
        return Path(user_config_dir(self.app_name))
    
    def config_auth(self) -> Path:
        return self.config_dir / "provider-config.yaml"
    
    def config_cookie(self) -> Path:
        return self.config_dir / "cookies.yaml"
    
    def _ensure_config_dir_exists(self) -> None:
        self.config_dir.mkdir(parents=True, exist_ok=True)
        
    def _ensure_config_files_exist(self) -> None:
        files = {
            self.config_auth(): "shodanx:\n    username:wiener\n    password:peter\n apikey:DIOBSFOUBGFOUBSOFBOFBF\n", #this is not valid key
            self.config_cookie(): "cookies:paste-ur-cookie-here"
        }
        for path, default_content in files.items():
            if not path.exists():
                path.write_text(default_content)


import aiofiles
import yaml
from fake_useragent import UserAgent
import sys
import socket

class ShodanxUtils:
    @staticmethod
    async def load_yaml(filename: str):
        async with aiofiles.open(filename, "r") as file:
            contents = await file.read()
            data = yaml.safe_load(contents)
            return data.get('shodanx', {}).get('username'), data.get('shodanx', {}).get('password')
        
    @staticmethod
    async def load_key(filename: str):
        async with aiofiles.open(filename, "r") as file:
            contents = await file.read()
            data = yaml.safe_load(contents)
            return data.get('shodanx', {}).get('apikey')

    @staticmethod
    async def load_json(filename: str):
        async with aiofiles.open(filename, "r") as file:
            contents = await file.read()
            data = yaml.safe_load(contents)
            return data.get('cookies', None)
        
    
    @staticmethod
    async def set_cookie(filename: str, cookie: str):
        new_data = {"cookies": cookie}
        async with aiofiles.open(filename, "w") as file:
            await file.write(yaml.dump(new_data, indent=4))

    @staticmethod
    async def set_auth(filename: str, username: str, password:str, apikey:str):
        new_data = {
            'shodanx': {
                'username': username,
                'password': password,
                'apikey': apikey
            }
        }
        async with aiofiles.open(filename, "w") as file:
            await file.write(yaml.dump(new_data, default_flow_style=False))
            
    @staticmethod
    async def async_reader(filename: str):
        try:
            async with aiofiles.open(filename, "r") as file:
                content = await file.readlines()
                return [line.strip() for line in content if line.strip()]
        except Exception:
            return None
            
    @staticmethod        
    def get_service_name(port:int):
        try:
            return socket.getservbyport(port, 'tcp')
        except Exception:
            try:
                return socket.getservbyport(port, 'udp')
            except Exception:
                return "unknown service"
            
    @staticmethod
    def random_useragent()-> str:
        return UserAgent().random
    
    def Exit(val = 0) -> None:
        sys.exit(val)
        
    @staticmethod
    def get_port_style_and_service(port:int, default_service="unknow service"):
        port_styles = {
        21:     ("bold red",     "FTP (plain-text credentials)"),
        22:     ("bold green",   "SSH (remote access)"),
        23:     ("bold red",     "Telnet (insecure remote access)"),
        25:     ("bold yellow",  "SMTP (email/spam vector)"),
        53:     ("bold cyan",    "DNS (recon)"),
        80:     ("bold green",        "HTTP (web)"),
        110:    ("bold yellow",  "POP3 (email)"),
        123:    ("bold yellow",       "NTP (time sync leak)"),
        135:    ("bold red",          "MS RPC (pivot risk)"),
        137:    ("bold red",          "NetBIOS Name (SMB)"),
        138:    ("bold red",          "NetBIOS Datagram"),
        139:    ("bold red",          "SMB (NetBIOS)"),
        143:    ("bold yellow",       "IMAP (email)"),
        161:    ("bold red",     "SNMP (info leak)"),
        162:    ("bold red",     "SNMP Trap (info leak)"),
        179:    ("yellow",       "BGP (routing protocol)"),
        389:    ("bold yellow",  "LDAP (auth exposure)"),
        443:    ("bold green",   "HTTPS (secure web)"),
        445:    ("bold red",     "SMB (EternalBlue, RCE risk)"),
        465:    ("bold green",        "SMTPS (secure email)"),
        514:    ("bold yellow",       "Syslog (log leak)"),
        587:    ("bold green",        "SMTP (submission)"),
        593:    ("bold red",          "RPC over HTTP"),
        631:    ("bold yellow",       "IPP (printer exposure)"),
        636:    ("bold green",        "LDAPS (secure directory)"),
        873:    ("bold yellow",       "rsync (unauth file access)"),
        3389:   ("bold yellow",  "RDP (remote desktop)"),
        5900:   ("bold yellow",  "VNC (unauth screen access)"),
        5800:   ("bold yellow",  "VNC over Web"),
        9001:   ("bold yellow",  "Tor ORPort (relay)"),
        1433:   ("bold red",     "MSSQL (DB exposure)"),
        1521:   ("bold red",     "Oracle DB"),
        3306:   ("bold red",     "MySQL (DB exposure)"),
        5432:   ("bold red",     "PostgreSQL (DB exposure)"),
        5984:   ("bold red",     "CouchDB (unauth access)"),
        6379:   ("bold red",     "Redis (unauth memory access)"),
        9200:   ("bold red",     "Elasticsearch (data exposure)"),
        11211:  ("bold red",     "Memcached (info leak)"),
        27017:  ("bold red",     "MongoDB (unauth access)"),
        8000:   ("bold cyan",         "Dev HTTP"),
        8080:   ("bold cyan",         "Proxy / Web API"),
        8443:   ("bold green",        "HTTPS Alt (dev)"),
        8888:   ("bold cyan",         "HTTP Admin"),
        5000:   ("bold cyan",         "Flask / API"),
        5601:   ("bold yellow",       "Kibana UI"),
        7001:   ("bold cyan",         "WebLogic console"),
        8008:   ("bold cyan",         "Android HTTP"),
        8983:   ("bold yellow",       "Apache Solr"),
        1883:   ("bold yellow",  "MQTT (IoT messaging)"),
        8883:   ("bold yellow",  "MQTTS (secure IoT)"),
        20000:  ("bold red",          "DNP3 (SCADA)"),
        44818:  ("bold red",          "EtherNet/IP (ICS)"),
        2049:   ("bold red",          "NFS (file exposure)"),
        5000:   ("bold yellow",       "Flask / Docker API"),
        6000:   ("bold red",          "X11 (screen capture risk)"),
        6666:   ("bold yellow",       "IRC (unfiltered)"),
        6667:   ("bold yellow",       "IRC"),
        8081:   ("bold cyan",         "Alt Web UI"),
        8889:   ("bold yellow",       "Jupyter Notebook"),
        2375:   ("bold red",          "Docker API (unauth)"),
        2379:   ("bold red",          "etcd (K8s config leak)"),
        2380:   ("bold red",          "etcd peer port"),
        6443:   ("bold yellow",       "Kubernetes API"),
        10250:  ("bold red",          "Kubelet (RCE risk)"),
        10255:  ("bold red",          "Kubelet ReadOnly"),
        1194:   ("bold green",        "OpenVPN"),
        1701:   ("bold green",        "L2TP"),
        500:    ("bold green",        "IPSec"),
        4500:   ("bold green",        "IPSec NAT-T"),
        5222:   ("bold yellow",       "XMPP (chat)"),
        5269:   ("bold yellow",       "XMPP Server"),
        6665:   ("bold yellow",       "IRC"),
        7000:   ("bold yellow",       "IRC"),
        7071:   ("bold yellow",       "Zimbra Admin"),
    }
        style, label = port_styles.get(port, ("bold yellow", default_service))
        return style, label




import httpx
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich.text import Text
from shodanx.modules.logger.logger import Logger
from typing import List,Dict
from rich import box


class ShodanFaviconMap():
    def __init__(self,session:httpx.AsyncClient,all:bool = False,top:int =50):
        self.logger = Logger()
        self.console = Console()
        self.session = session
        self.all = all
        self.top = top
        self._shodan_url = "https://faviconmap.shodan.io/data/favicons.json"
        
    async def search(self) -> list:
        try:
            response: httpx.Response = await self.session.request("GET", self._shodan_url, timeout=30)
            if response.status_code != 200:
                return []
            data = response.json()
            self.display_favicon_ranking(data)
            return data
        except Exception as e:
            self.logger.warn(f"Error occured in the favicon map search due to: {e}")
            return []
        
    def display_favicon_ranking(self, data: List[Dict]) -> None:

        sorted_data = sorted(data, key=lambda x: x["count"], reverse=True)
        display_data = sorted_data if self.all else sorted_data[:self.top]
        
        table = Table(
            title="🔍 [bold #00ff41]ShodanX Favicon Intelligence[/] [bold #ff00ff]::[/] [bold #00ffff]Hash Rankings[/]",
            box=box.DOUBLE_EDGE,
            border_style="bold #ff00ff",
            header_style="bold #00ffff",
            style="bold #00ff41 on #0a0a0a",
            show_lines=True
        )
        
        table.add_column("[bold #ff5555]Rank[/]", justify="right", style="bold #00ffff")
        table.add_column("[bold #ff5555]Favicon Hash[/]", style="bold #ff00ff")
        table.add_column("[bold #ff5555]Host Count[/]", justify="right", style="bold #00ff41")
        table.add_column("[bold #ff5555]Shodan Query[/]", style="bold #ffff00", overflow="fold")
        
        for rank, entry in enumerate(display_data, start=1):
            hash_val = str(entry["hash"])
            count = entry["count"]
            query_url = f"https://www.shodan.io/search?query=http.favicon.hash:{hash_val}"
            
            if count > 10000:
                count_style = "[blink bold red]"
                rank_style = "[bold red]"
            elif count > 5000:
                count_style = "[bold red]"
                rank_style = "[bold yellow]"
            elif count > 1000:
                count_style = "[bold yellow]"
                rank_style = "[bold #00ffff]"
            else:
                count_style = "[bold #00ff41]"
                rank_style = "[bold white]"
            
            formatted_count = f"{count_style}{count:,}[/]"
            
            table.add_row(
                f"{rank_style}{rank}[/]",
                f"[bold magenta]{hash_val}[/]",
                formatted_count,
                f"[link={query_url}][bold cyan]shodan search[/][/]"
            )
        
        total_hashes = len(data)
        total_hosts = sum(entry["count"] for entry in data)
        top_percentage = (sum(entry["count"] for entry in display_data) / total_hosts * 100)
        
        summary_table = Table(
            box=box.SIMPLE_HEAVY,
            show_header=False,
            style="bold #00ff41"
        )
        summary_table.add_column("Metric", style="bold #00ffff")
        summary_table.add_column("Value", style="bold #ff5555")
        
        summary_table.add_row("Total Unique Hashes", f"[bold white]{total_hashes:,}[/]")
        summary_table.add_row("Total Hosts Mapped", f"[bold white]{total_hosts:,}[/]")
        summary_table.add_row("Top Coverage", f"[bold white]{top_percentage:.1f}%[/]")
        
        self.console.print(Panel(
            table,
            title="[bold #ff00ff] FAVICON THREAT INTELLIGENCE [/]",
            border_style="bold #00ffff",
            padding=(0, 2)
        ))
        
        self.console.print(Panel(
            summary_table,
            title="[bold #ffff00] STATISTICS SUMMARY [/]",
            border_style="bold #ff5555",
            width=60
        ))
        
        self.console.print(
            Text.from_markup(
                "[bold #00ff41]Tip: [white]Click on Shodan query links to view hosts with matching favicon hashes[/]"
            ),
            justify="center"
        )


import httpx
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.table import Table
from rich.rule import Rule
from rich.align import Align

from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils


class ShodanxMapsOSINT:
    def __init__(self, session: httpx.AsyncClient) -> None:
        self.session = session
        self.console = Console()
        self.logger = Logger()
        self._shodan_utils = ShodanxUtils()
        self._shodan_url = "https://maps.shodan.io/_search"
        self.headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Alt-Used": "maps.shodan.io",
            "Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "cross-site",
            "Priority": "u=0, i",
            "Pragma": "no-cache",
            "Cache-Control": "no-cache"
        }

    def _generate_maps_link(self, latitude, longitude):
        return f"https://www.google.com/maps?q={latitude},{longitude}"

    async def search(self, cookies_file: str, place: str) -> dict:
        try:
            cookies = await self._shodan_utils.load_json(cookies_file)
            if not cookies:
                self.logger.warn("No cookies found. Please configure your Shodan keys.")
                return {}

            self.headers["Cookie"] = cookies
            self.headers["User-Agent"] = self._shodan_utils.random_useragent()
            params = {"q": place}

            response: httpx.Response = await self.session.request("GET", self._shodan_url, timeout=30, headers=self.headers, params=params)
            if response.status_code != 200:
                self.logger.warn(f"Received non-200 status: {response.status_code}")
                return {}

            data = response.json()
            self.display_maps_osint_results(data)
            return data

        except Exception as e:
            self.logger.warn(f"Error occurred in Shodan Maps OSINT: {e}")
            return {}

    def display_maps_osint_results(self, data):
        self.console.print(
            Panel(
                Align.center(
                    Text.from_markup(
                        f"[bold magenta]:satellite: ShodanX:zap: Advanced Recon[/bold magenta]\n"
                        f"[bold cyan]🧠 Total Matches:[/bold cyan] [bold white]{data['total']:,}[/bold white] | "
                        f"[bold green]Displayed:[/bold green] {len(data['matches'])}\n"
                        f"[dim]🚀 Think like an attacker. Visualize like a hacker.[/dim]"
                    ),
                    vertical="middle"
                ),
                title="🛰️ [bright_white]MAPS INTELLIGENCE[/bright_white]",
                border_style="bold bright_magenta",
            )
        )

        for idx, match in enumerate(data['matches'], 1):
            ip = match['ip_str']
            port = match['port']
            loc = match['location']

            base_service = self._shodan_utils.get_service_name(port)
            port_style, service = self._shodan_utils.get_port_style_and_service(port, base_service)

            country = f"{loc.get('country_name', 'Unknown')} ({loc.get('country_code', '--')})"
            city = loc.get("city", "N/A")
            region = loc.get("region_code", "N/A")
            lat = loc.get("latitude", 0)
            lon = loc.get("longitude", 0)
            coords = f"{lat:.4f}, {lon:.4f}"
            maps_link = self._generate_maps_link(lat, lon)

            ipinfo_link = f"https://ipinfo.io/{ip}"
            shodan_link = f"https://www.shodan.io/host/{ip}"

            host_table = Table.grid(padding=(0, 2))
            host_table.add_column(justify="right", style="cyan", no_wrap=True)
            host_table.add_column(style="bold white")

            host_table.add_row("🧠 IP", f"[bold cyan]{ip}[/bold cyan]")
            host_table.add_row("🔌 Port", f"[{port_style}]{port}[/{port_style}] → [bold green]{service}[/bold green]")
            host_table.add_row("📍 Location", f"[yellow]{city}, {region}[/yellow] — [green]{country}[/green]")
            host_table.add_row("🌐 Coords", f"[white]{coords}[/white]")
            host_table.add_row("🗺️  Map", f"[link={maps_link}]Google Maps[/link]")
            host_table.add_row("🧰 OSINT", f"[link={ipinfo_link}]ipinfo[/link] | [link={shodan_link}]shodan[/link]")

            self.console.print(
                Panel(
                    host_table,
                    title=f"[bold cyan]# {idx} → Host OSINT Map Data[/bold cyan]",
                    border_style="bright_blue"
                )
            )

        self.console.print(Rule(style="bright_magenta"))
        self.console.print(
            Panel(
                "[bold white]Additional Recon Resources:[/bold white]\n"
                "- [link=https://www.shodan.io/]Shodan Search[/link] for more host fingerprints\n"
                "- [link=https://viz.greynoise.io/]GreyNoise Viz[/link] to profile noise vs. target\n"
                "- [link=https://censys.io/]Censys[/link] for certificate and device scans\n"
                "- [link=https://www.onyphe.io/]Onyphe[/link] for cyber threat context\n",
                title="[bold green]🔎 Hacker’s Recon Panel[/bold green]",
                border_style="green"
            )
        )



import httpx
from bs4 import BeautifulSoup,XMLParsedAsHTMLWarning, MarkupResemblesLocatorWarning
import warnings
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
import getpass
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.prompt import Prompt


class ShodanAuth():
    def __init__(self,auth_yaml: str, cookie_json: str ,session: httpx.AsyncClient = None):
        self.session = session
        self.logger = Logger()
        self._auth_url = "https://account.shodan.io/login"
        self._shodan_utils = ShodanxUtils()
        self.auth_file = auth_yaml
        self.cookie_file = cookie_json
        self.console =  Console()
        self.cookie = None
        
    async def get_csrf_token(self) -> str | None:
        response : httpx.Response = await self.session.request(
            "GET", 
            self._auth_url, 
            headers={"User-Agent": self._shodan_utils.random_useragent()}, 
            timeout=30, 
            follow_redirects=True)
        
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", category=UserWarning)
            warnings.filterwarnings('ignore', category=XMLParsedAsHTMLWarning)
            warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)
            soup = BeautifulSoup(response.text, "lxml")
            csrf_token_input = soup.find('input', {'name': 'csrf_token'})
            if csrf_token_input and csrf_token_input.get('value'):
                return csrf_token_input.get('value')
            else:
                return None
            
    async def login(self) -> str:
        username,password = await self._shodan_utils.load_yaml(self.auth_file)
        if  username is None or password is None:
            self.logger.warn("missing username or password of shodan, please update in the shodanx configuration file!")
            exit()
        csrf_token = await self.get_csrf_token()
        if not csrf_token:
            self.logger.warn("Unable to get CSRF Token!")
            csrf_token = "ef4d11775524c201f89e15aa7f489492264ad78c"
        body = {
            "username": username,
            "password": password,
            "grant_type":	password,
            "continue":	"https://account.shodan.io/",
            "csrf_token": csrf_token
        }
        headers = {
            'User-Agent': ShodanxUtils.random_useragent(),
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
            'Accept-Language': 'en-US,en;q=0.5',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Origin': 'https://account.shodan.io',
            'Connection': 'keep-alive',
            'Referer': 'https://account.shodan.io/login',
            'Cookie': '_ga=GA1.2.477518759.1715975591; _ga_BNT9JE7KQX=GS2.2.s1746366123$o3$g0$t1746366123$j0$l0$h0; cf_clearance=3rC2dBEmiwC6hnNv1wMQmPQ1klLi2HM7f.Q.RVDLI4Q-1720690157-1.0.1.1-4NSsfAl.vKpLpYH.wSqyfInQTMTyYGoHkq4by9h6CR34d661r3jJRNT9bzhgs9qlp6DAIhEpRsz9vjnG2or2qQ; _gid=GA1.2.1420983711.1746355713; session=avmZ3iUoZP4PiU6Q6dhukKde2sDMT1C6cyJPW8_i-IWXExQzy_RL0ag4lNTmWIKfSgVF_w4W14blHN_cyiRrx4AFlXYAAAAAAAAASpqWF2hHQdoF4NNPMpl9lCiMDGNvbnRpbnVlX3VybJSMGmh0dHBzOi8vYWNjb3VudC5zaG9kYW4uaW8vlIwHX2NzcmZ0X5SMKGRjZjFkNzM1NDYyZmU2ZmJmMDNhZjU3MGIwMjhkMTNiNDcwYjliZDCUdYeULg',
            'Upgrade-Insecure-Requests': '1',
            'Sec-Fetch-Dest': 'document',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-User': '?1',
            'Priority': 'u=0, i',
            'Pragma': 'no-cache',
            'Cache-Control': 'no-cache',
            'TE': 'trailers'
        }
        response : httpx.Response = await self.session.request(
            "POST", 
            self._auth_url, 
            headers=headers, 
            timeout=30,
            data=body,
            follow_redirects=True
        )
        
        if response.status_code != 302 and "Invalid username or password" in response.text:
            self.logger.warn("invalid credentials detected, please config the valid credentials! or set cookie manually!")
            return None
                
        cookies = response.cookies
        if "polito" in cookies and "session" in cookies:
            cookie = f"polito={cookies['polito']}; session={cookies['session']}"
            return cookie
        else:
            self.logger.warn("invalid cookie, please set the cookie manually and restart!")
            return None
        
    async def valid_session(self) -> bool:
        cookies = await self._shodan_utils.load_json(self.cookie_file)
        if cookies is None:
            return False
        
        headers = {
            'User-Agent': ShodanxUtils.random_useragent(),
            'Cookie': str(cookies),
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
            'Accept-Language': 'en-US,en;q=0.5',
            'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Origin': 'https://account.shodan.io',
            'Connection': 'keep-alive',
            'Referer': 'https://account.shodan.io/login',
            'Upgrade-Insecure-Requests': '1',
            'Sec-Fetch-Dest': 'document',
            'Sec-Fetch-Mode': 'navigate',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-User': '?1',
            'Priority': 'u=0, i',
            'Pragma': 'no-cache',
            'Cache-Control': 'no-cache',
            'TE': 'trailers'
        }
        response: httpx.Response = await self.session.request("GET", "https://account.shodan.io/", timeout=30, headers=headers)
        if response.status_code != 200:
            return False
        return True
    
    async def shell(self) -> dict:
        console = Console()
        
        welcome_panel = Panel.fit(
            Text("Welcome to Shodanx Auth Configuration Mode\nPlease Config your Shodan Credentials for ShodanX", 
                 style="bold green"),
            title="[bold blue]ShodanX Authentication Setup[/]",
            border_style="bright_blue",
            padding=(1, 4),
            style="bold white"
        )
        console.print(welcome_panel)

        try:
            username = input("Enter your Shodan username: ")
            password = getpass.getpass("Enter your Shodan password (hidden): ")
            api_key = getpass.getpass("Enter your Shodan API key (hidden): ")
            
            if not all([username, password, api_key]):
                console.print("[bold red]Error: All fields are required![/]")
                return None

            confirmation_text = Text()
            confirmation_text.append(f"Username : {username}\n", style="bold cyan")
            confirmation_text.append(f"Password : {'*' * len(password)}\n", style="bold magenta")
            confirmation_text.append(f"API Key  : {'*' * len(api_key)}\n\n", style="bold yellow")
            confirmation_text.append("Please verify these details are correct.", style="bold green")

            confirmation_panel = Panel.fit(
                confirmation_text,
                title="[bold green]Confirmation[/]",
                border_style="bright_green",
                padding=(1, 4)
            )
            console.print(confirmation_panel)

            confirm = input("Are these details correct? (y/n): ").lower()
            if confirm != 'y':
                console.print("[bold yellow]Configuration aborted[/]")
                return None

            return {
                'username': username,
                'password': password,
                'apikey': api_key,
            }
        except KeyboardInterrupt:
            console.print("\n[bold red]Input cancelled by user[/]")
            return None
        except Exception as e:
            console.print(f"[bold red]Error during input: {e}[/]")
    
    async def shell_login(self):
        console = Console()
        welcome_panel = Panel.fit(
            Text("Welcome to Shodanx Login Mode", 
                 style="bold green"),
            title="[bold blue]ShodanX Login Setup[/]",
            border_style="bright_blue",
            padding=(1, 4),
            style="bold white"
        )
        console.print(welcome_panel)
        try:
            ask = Prompt()
            console.print(f"[bold green]Please make sure paste cookies without spaces[/]")
            cookie = ask.ask("Enter your active sesson cookie (hidden) ", password=True)
            if len(cookie) == 0:
                console.print(f"[bold red]Please enter a valid session cookies![/]")
                return None
            return cookie
        except Exception as e:
            console.print(f"[bold red]Error during input: {e}[/]")
            return None



import httpx
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
from rich import box

class ShodanxCVEDB:
    def __init__(
        self,
        session: httpx.AsyncClient,
        product: str = None,
        cpe23: str = None,
        count: bool = False,
        is_kev: bool = False,
        sort_by_epss: bool = False,
        skip: int = 0,
        limit: int = 1000,
        start_date: str = None,
        end_date: str = None,
        verbose: bool = False,
    ):
        self.session = session
        self.product = product
        self.cpe23 = cpe23
        self.count = count
        self.is_kev = is_kev
        self.sort_by_epss = sort_by_epss
        self.skip = skip
        self.limit = limit
        self.start_date = start_date
        self.end_date = end_date
        self.verbose = verbose
        self.console = Console()
        self.logger = Logger()
        self.utils = ShodanxUtils
        self._shodan_url = "https://cvedb.shodan.io"

    async def search(self,cpe=None,product=None) -> dict:
        if self.product and self.cpe23:
            self.logger.error("You can only specify one of 'product' or 'cpe23'.")
            return {}
        params = {
            "is_kev": self.is_kev,
            "skip": self.skip,
            "limit": self.limit,
        }
        if self.start_date:
            params["start_date"] = self.start_date
        if self.end_date:
            params["end_date"] = self.end_date
        if cpe:
            params["cpe23"] = cpe
        if product:
            params["product"] = product
        try:
            url = f"{self._shodan_url}/cves"
            response: httpx.Response = await self.session.request("GET",url,params=params, timeout=30)
            if response.status_code != 200:
                return {}
            data = response.json()
            self._display_cve_all_table(data.get("cves", []))
            return data
        except Exception as e:
            self.logger.warn(f"Error occured in the cve search module due to: {e}, {type(e)}")
            return {}

    async def search_cve_by_id(self,cveid: str) -> dict:
        try:
            request_url = f"{self._shodan_url}/cve/{cveid}"
            response : httpx.Response = await self.session.request("GET", request_url, timeout=30)
            if response.status_code != 200:
                return {}
            data = response.json()
            self._display_cve_details(data)
            return data
        except Exception as e:
            self.logger.warn(f"Error occured in the id cve search module due to: {e}")
            return {}
        
    async def search_cve_by_cpe(self,cpe:str) -> dict:
        try:
            params = {
            "cpe23":cpe,
            "is_kev": self.is_kev,
            "skip": self.skip,
            "limit": self.limit,
            }
            if self.start_date:
                params["start_date"] = self.start_date
            if self.end_date:
                params["end_date"] = self.end_date
            request_url = f"{self._shodan_url}/cves"
            response: httpx.Response =  await self.session.request("GET", request_url, timeout=30, params=params)
            if response.status_code != 200:
                return {}
            data = response.json()
            self._display_cve_all_table(data)
            return data
        except Exception as e:
            self.logger.warn(f"Error occured in the cpe search module due to: {e}")
            return {}
    
    
        
    def _display_cve_all_table(self, cves: list):
        if not cves:
                self.console.print("[bold yellow]No CVEs found.[/bold yellow]")
                return
            
        table = Table(
                title="[bold white]Shodanx CVE Database Results[/bold white]",
                box=box.ROUNDED,
                style="bold white",
                show_lines=True, 
                header_style="bold cyan",
                row_styles=["none"]
                )
    
        table.add_column("CVE ID", style="bold cyan", no_wrap=True, min_width=12)
        table.add_column("Summary", style="bold white", min_width=40, overflow="fold")
        table.add_column("CVSS", style="bold red", justify="center", width=6)
        table.add_column("EPSS", style="bold green", justify="center", width=6)
        table.add_column("Exploited", style="bold magenta", justify="center", width=10)
        table.add_column("References", style="bold yellow", min_width=30, overflow="fold")
        table.add_column("Published", style="bold blue", no_wrap=True, width=20)

        for cve in cves:
            references = cve.get("references", [])
            if len(references) > 3:
                refs = "\n".join(references[:3]) + f"\n[...+{len(references)-3} more]"
            else:
                refs = "\n".join(references) if references else "N/A"

            summary = cve.get("summary", "N/A")
            

            table.add_row(
                cve.get("cve_id", "N/A"),
                summary,
                str(cve.get("cvss", "N/A")),
                str(cve.get("epss", "N/A")),
                "✅" if cve.get("kev") else "❌",
                refs,
                cve.get("published_time", "N/A")
            )
    
        self.console.print(Panel.fit(
            table,
            title="[bold]CVE Database Results[/bold]",
            border_style="blue",
            padding=(1, 2)
        ))
        
    def _display_cve_details(self, cve: dict):
        table = Table(title=f"CVE Details: {cve.get('cve_id', 'N/A')}", style="bold white",box=box.ROUNDED,show_lines=True, header_style="bold cyan",row_styles=["none"])
        table.add_column("Field", style="bold magenta", no_wrap=True)
        table.add_column("Data", style="bold cyan")
        table.add_row("CVE ID", cve.get("cve_id", "N/A"),style="bold white")
        table.add_row("Summary", cve.get("summary", "N/A"),style="bold cyan")
        table.add_row("CVSS (Overall)", str(cve.get("cvss", "N/A")),style="bold yellow")
        table.add_row("EPSS", str(cve.get("epss", "N/A")),style="bold cyan")
        table.add_row("Ranking EPSS", str(cve.get("ranking_epss", "N/A")),style="bold cyan")
        table.add_row("KEV (Exploited)", "✅ Yes" if cve.get("kev") else "❌ No",style="bold white")
        table.add_row("Proposed Action", cve.get("propose_action", "N/A"), style="bold magenta")
        table.add_row("Ransomware Campaign", cve.get("ransomware_campaign", "N/A"),style="bold white")
        table.add_row("Published Time", cve.get("published_time", "N/A"),style="bold green")
        
        references = "\n".join(cve.get("references", [])) or "N/A"
        table.add_row("References", references,style="bold cyan")
        cpes = "\n".join(cve.get("cpes", [])) or "N/A"
        table.add_row("CPEs", cpes,style="bold blue")

        self.console.print(table)
        


from setuptools import setup, find_packages

with open("README.md", "r") as streamr:
    long_description = streamr.read()

setup(
    name='shodanx',
    version='1.1.1',
    author='D.Sanjai Kumar',
    author_email='bughunterz0047@gmail.com',
    description='ShodanX is a terminal-powered recon and OSINT tool built on top of the Shodan Services.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RevoltSecurities/ShodanX",
    packages=find_packages(),
    install_requires=[
        'aiodns>=3.2.0',
        'aiofiles>=24.1.0',
        'appdirs>=1.4.4',
        'art>=6.5',
        'beautifulsoup4>=4.13.4',
        'click>=8.2.0',
        'colorama>=0.4.6',
        'fake_useragent>=2.0.3',
        'httpx>=0.28.1',
        'PyYAML>=6.0.2',
        'rich>=14.0.0',
        'setuptools>=78.1.0',
        'lxml>=5.3.2',
        'bs4>=0.0.2',
    ],
    entry_points={
        'console_scripts': [
            'shodanx = shodanx.shodanx:cli'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import httpx
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
from rich.panel import Panel
from rich.console import Console
from rich.rule import Rule
from rich.tree import Tree
from collections import defaultdict
from typing import Dict, Any


class ShodanxTrendsDB:
    def __init__(self, yaml_file: str, query: str, session: httpx.AsyncClient, facet: str = "ip") -> None:
        self.yaml_auth = yaml_file
        self.query = query
        self.session = session
        self.facet = facet
        self.logger = Logger()
        self._shodan_utils = ShodanxUtils()
        self._shodan_url = f"https://trends.shodan.io/api/v1/search?query={self.query}&facets={self.facet}"
        self.console = Console()

    def _generate_bar(self, count, max_count, length=30):
        filled_len = int(length * count // max_count) if max_count else 0
        bar = "█" * filled_len + "░" * (length - filled_len)
        return f"[green]{bar}[/green]"

    async def search_trends(self) -> dict:
        try:
            apikey = await self._shodan_utils.load_key(self.yaml_auth)
            if not apikey:
                self.logger.warn("No Shodan API key found for Shodan Trends search, please configure a valid API key")
                return {}
            url = f"{self._shodan_url}&key={apikey}"
            response: httpx.Response = await self.session.request("GET", url, timeout=30)
            if response.status_code != 200:
                self.logger.warn(f"Shodan Trends returned non-200 status code: {response.status_code}")
                return {}
            data = response.json()
            self.display_trends(data)
            return data
        except Exception as e:
            self.logger.warn(f"Error occurred in the trends search module: {e}")

    def display_trends(self, data: Dict[str, Any]) -> None:
        matches = data.get("matches", [])
        facet_data = data.get("facets", {}).get(self.facet, [])

        if not matches and not facet_data:
            self.console.print(f"[bold red]❌ No trend data available for facet: {self.facet}[/bold red]")
            return

        total_count = data.get("total", 0)
        max_count = max((m["count"] for m in matches), default=1)

        overview = Panel.fit(
            f"[bold bright_white]🔍 Query:[/bold bright_white] {self.query}\n"
            f"[bold bright_white]📊 Total Records:[/bold bright_white] {total_count:,}\n"
            f"[bold bright_white]🗓️ Time Range:[/bold bright_white] {matches[0]['month']} → {matches[-1]['month']} ({len(matches)} months)\n"
            f"[bold bright_white]🎯 Facet:[/bold bright_white] [cyan]{self.facet.upper()}[/cyan]",
            title="[bold blue]Shodan Trends Overview[/bold blue]",
            border_style="bright_magenta",
            padding=(1, 2)
        )
        self.console.print(overview)

        if matches:
            self.console.print(Rule("[bold cyan]📈 Monthly Activity Overview[/bold cyan]"))
            prev_count = None
            for match in matches:
                month = match["month"]
                count = match["count"]

                change_text = ""
                if prev_count is not None and prev_count != 0:
                    change = ((count - prev_count) / prev_count) * 100
                    if change > 0:
                        change_text = f"[green]🟢 +{change:.1f}%[/green]"
                    elif change < 0:
                        change_text = f"[red]🔻 {change:.1f}%[/red]"
                    else:
                        change_text = f"[yellow]⚪ 0.0%[/yellow]"

                bar = self._generate_bar(count, max_count)
                self.console.print(
                    f"[bold white]{month}[/bold white] ─ {count:,} {change_text}\n{bar}"
                )
                prev_count = count
            self.console.print()

        if facet_data:
            self.console.print(Rule(f"[bold magenta]📊 {self.facet.upper()} Distribution[/bold magenta]"))
            yearly_facet_data = defaultdict(list)
            for entry in facet_data:
                year = entry["key"][:4]
                yearly_facet_data[year].append(entry)

            distribution_tree = Tree(f"[bold underline]📊 {self.facet.upper()} Distribution[/bold underline]")
            for year, entries in sorted(yearly_facet_data.items()):
                year_branch = distribution_tree.add(f"[bold magenta]{year}[/bold magenta]")

                months = defaultdict(list)
                for entry in entries:
                    month = entry["key"][5:]  
                    months[month].append(entry)
                for month, month_entries in sorted(months.items()):
                    month_branch = year_branch.add(f"[bold yellow]{month}[/bold yellow]")

                    for entry in month_entries:

                        total_month_count = next(
                            (m["count"] for m in matches if m["month"] == entry["key"]),
                            sum(v["count"] for v in entry["values"])
                        )
                        for val in entry["values"]:
                            domain = str(val["value"])
                            count = val["count"]
                            percentage = (count / total_month_count * 100) if total_month_count > 0 else 0
                            month_branch.add(
                                f"[green]{domain}[/green] ─ [yellow]{count:,}[/yellow] ([cyan]{percentage:.1f}%[/cyan])"
                            )
            self.console.print(distribution_tree)
            self.console.print()

        info_panel = Panel.fit(
            f"[bold white]🧾 Displayed {len(matches)} monthly totals and {sum(len(m['values']) for m in facet_data)} {self.facet.upper()} values[/bold white]\n"
            f"[bold white]Available facets:[/bold white] [italic cyan]{', '.join(data.get('facets', {}).keys())}[/italic cyan]",
            title="[green]Data Summary[/green]",
            border_style="green"
        )
        self.console.print(info_panel)



import aiofiles
import json
from shodanx.modules.logger.logger import Logger
from typing import Any


class ShodanxSave():
    def __init__(self, logger: Logger):
        self.logger = logger
    
    async def save(self,filename: str, content: Any,jsonize:bool = False) -> None:
        try:
            async with aiofiles.open(filename, "a") as streamw:
                if jsonize:
                    await streamw.write(json.dumps(content, indent=4)+ "\n")
                else:
                    await streamw.write(content + '\n')
        except Exception as e:
            self.logger.warn(f"error occured in shodanx save module due to: {e}, {type(e)}, {content}")


from colorama import Fore, Style, init
import sys
import random
from datetime import datetime
from enum import Enum, auto
import random

init()

class LogLevel(Enum):
    DEBUG = auto()
    VERBOSE = auto()
    INFO = auto()
    WARN = auto()
    ERROR = auto()
    CRITICAL = auto()
    NONE = auto()  

class Logger:
    def __init__(self, name: str = None, colored: bool = True, level: LogLevel = LogLevel.INFO):
        self.colored = colored
        self.name = name
        self.level = level
        self.red = Fore.RED
        self.green = Fore.GREEN
        self.yellow = Fore.YELLOW
        self.blue = Fore.BLUE
        self.magenta = Fore.MAGENTA
        self.cyan = Fore.CYAN
        self.white = Fore.WHITE
        self.bold = Style.BRIGHT
        self.reset = Style.RESET_ALL
        self.color_list = [self.red, self.green, self.yellow, self.blue, self.magenta, self.cyan, self.white]
        self.random_color = random.choice(self.color_list)

        
        self.colors = {
            LogLevel.INFO: Fore.BLUE,
            LogLevel.WARN: Fore.YELLOW,
            LogLevel.ERROR: Fore.RED,
            LogLevel.VERBOSE: Fore.GREEN,
            LogLevel.DEBUG: Fore.CYAN,
            LogLevel.CRITICAL: Fore.MAGENTA + Style.BRIGHT,
            'time': Fore.BLUE
        }
        
        self.level_names = {
            LogLevel.DEBUG: "DEBUG",
            LogLevel.VERBOSE: "VERBOSE",
            LogLevel.INFO: "INFO",
            LogLevel.WARN: "WARN",
            LogLevel.ERROR: "ERROR",
            LogLevel.CRITICAL: "CRITICAL"
        }
    
    def set_level(self, level: LogLevel):
        self.level = level
    
    def should_log(self, level: LogLevel) -> bool:
        return level.value >= self.level.value and self.level != LogLevel.NONE
    
    def _get_timestamp(self):
        return datetime.now().strftime('%H:%M:%S')
    
    def _format_level(self, level: LogLevel):
        if not self.colored:
            return self.level_names.get(level, "LOG")
        color = self.colors.get(level, Fore.WHITE)
        return f"{Style.BRIGHT}{color}{self.level_names.get(level, 'LOG')}{Style.RESET_ALL}"
    
    def _format_name(self):
        if not self.name:
            return ""
        if not self.colored:
            return f"[{self.name}]"
        return f"[{Style.BRIGHT}{Fore.MAGENTA}{self.name}{Style.RESET_ALL}]"
    
    def _format_timestamp(self):
        if not self.colored:
            return f"[{self._get_timestamp()}]"
        return f"[{Style.BRIGHT}{self.colors['time']}{self._get_timestamp()}{Style.RESET_ALL}]"
    
    def _log(self, level: LogLevel, message: str):
        if not self.should_log(level):
            return
            
        timestamp = self._format_timestamp()
        name = self._format_name()
        level_str = self._format_level(level)
        
        if self.colored:
            message = f"{Style.BRIGHT}{Fore.WHITE}{message}{Style.RESET_ALL}"
            
        log_line = f"{timestamp} {name} [{level_str}]: {message}"
        print(log_line, file=sys.stderr)
    
    def debug(self, message: str):
        self._log(LogLevel.DEBUG, message)
        
    def verbose(self, message: str):
        self._log(LogLevel.VERBOSE, message)
        
    def info(self, message: str):
        self._log(LogLevel.INFO, message)
        
    def warn(self, message: str):
        self._log(LogLevel.WARN, message)
        
    def error(self, message: str):
        self._log(LogLevel.ERROR, message)
        
    def critical(self, message: str):
        self._log(LogLevel.CRITICAL, message)
        
    def custom(self, level_name: str, message: str, color: str = Fore.WHITE):
        if not self.colored:
            level_str = level_name.upper()
        else:
            level_str = f"{Style.BRIGHT}{color}{level_name.upper()}{Style.RESET_ALL}"
            
        timestamp = self._format_timestamp()
        name = self._format_name()
        
        if self.colored:
            message = f"{Style.BRIGHT}{Fore.WHITE}{message}{Style.RESET_ALL}"
            
        log_line = f"{timestamp}{name}[{level_str}]: {message}"
        print(log_line, file=sys.stderr)
    
    def banner(self, banner: str):
        print(banner, file=sys.stderr)
        
    def stdin(self, message: str):
        print(message)


import httpx
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from shodanx.modules.logger.logger import Logger
from shodanx.modules.shodanutils.shodanutils import ShodanxUtils
from typing import Dict,List
from rich import box

class ShodanExposureDB():
    def __init__(self,country: str, session:httpx.AsyncClient):
        self.logger = Logger()
        self.console = Console()
        self._shodan_utils = ShodanxUtils()
        self.country = country.lower()
        self._shodan_url = "https://shodan.nyc3.digitaloceanspaces.com/exposure-data"
        self.session = session
        self.countries = {
            "united states": "US.json",
            "australia": "AU.json",
            "austria": "AT.json",
            "bangladesh": "BD.json",
            "bahamas": "BS.json",
            "belarus": "BY.json",
            "belgium": "BE.json",
            "brazil": "BR.json",
            "bulgaria": "BG.json",
            "canada": "CA.json",
            "caribbean": "KY.json", 
            "chile": "CL.json",
            "china": "CN.json",
            "costa rica": "CR.json",
            "denmark": "DK.json",
            "finland": "FI.json",
            "france": "FR.json",
            "germany": "DE.json",
            "guatemala": "GT.json",
            "hong kong": "HK.json",
            "hungary": "HU.json",
            "iceland": "IS.json",
            "india": "IN.json",
            "indonesia": "ID.json",
            "israel": "IL.json",
            "iran": "IR.json",
            "ireland": "IE.json",
            "italy": "IT.json",
            "japan": "JP.json",
            "kazakhstan": "KZ.json",
            "kenya": "KE.json",
            "kyrgyzstan": "KG.json",
            "malaysia": "MY.json",
            "mexico": "MX.json",
            "montenegro": "ME.json",
            "morocco": "MA.json",
            "mozambique": "MZ.json",
            "netherlands": "NL.json",
            "norway": "NO.json",
            "panama": "PA.json",
            "philippines": "PH.json",
            "poland": "PL.json",
            "portugal": "PT.json",
            "romania": "RO.json",
            "russia": "RU.json",
            "serbia": "RS.json",
            "slovakia": "SK.json",
            "slovenia": "SI.json",
            "spain": "ES.json",
            "south africa": "ZA.json",
            "south korea": "KR.json",
            "sweden": "SE.json",
            "switzerland": "CH.json",
            "taiwan": "TW.json",
            "turks & caicos": "TC.json",
            "ukraine": "UA.json",
            "venezuela": "VE.json",
            "vietnam": "VN.json"
        }

    def get_json_filename(self) -> str:
        return self.countries.get(self.country, None)
    
    async def search_exposure(self) -> dict:
        try:
            country_file = self.get_json_filename()
            if country_file is None:
                self.logger.warn(f"{self.country} is not found in shodan exposure db, please continue search with other countries.")
                return {}
            url = f"{self._shodan_url}/{country_file}"
            response: httpx.Response = await self.session.request("GET", url, timeout=30)
            if response.status_code != 200:
                return {}
            data = response.json()
            self.display_reports(data)
            return data
        except Exception as e:
            self.logger.warn(f"Error occured in the exposure search due to: {e}")
            return {}

    def _create_threat_overview(self, data: Dict) -> Table:
        table = Table(
            title=f"🔓 [bold #00ff41]ShodanX Threat Matrix[/] | [bold #ff00ff]{self.country}[/]",
            box=box.DOUBLE_EDGE,
            border_style="bold #ff00ff",
            header_style="bold #00ffff",
            style="bold #00ff41 on #0a0a0a"
        )
        table.add_column("[bold #ff5555]Threat Vector[/]", style="bold #00ffff")
        table.add_column("[bold #ff5555]Exposure Count[/]", justify="right", style="bold #ff00ff")

        table.add_row(
            Panel("[blink bold white] CRITICAL THREATS [/]", style="on #330000"),
            "",
            style="on #330000"
        )
        table.add_row("💀 Compromised DBs", f"[blink bold red]{data.get('numCompromisedDB', 0)}[/]")
        table.add_row("☠️ ICS Devices", f"[bold red]{data.get('numIcs', 0)}[/]")
        table.add_row("🔥 BlueKeep", f"[blink bold red]{data.get('numBluekeep', 0)}[/]")
        table.add_row("💣 EternalBlue", f"[blink bold red]{data.get('numEternalblue', 0)}[/]")

        table.add_row(
            Panel("[bold yellow] SECURITY WARNINGS [/]", style="on #333300"),
            "",
            style="on #333300"
        )
        table.add_row("📷 Exposed Webcams", f"[bold yellow]{data.get('numOpenWebcams', 0)}[/]")
        table.add_row("🛡️ Top CVE", f"[bold yellow]{data.get('vuln', 'N/A')}[/]")

        return table

    def _create_port_analysis(self, ports_data: List[List[str]]) -> Table:
        """Create detailed port analysis table using ShodanxUtils"""
        table = Table(
            title="[bold #00ffff]Port Vulnerability Analysis[/]",
            box=box.ROUNDED,
            border_style="bold #00ffff",
            show_lines=True,
            style="bold #00ff41 on #0a0a0a"
        )
        table.add_column("#", style="bold #ff5555", justify="right")
        table.add_column("Port", style="bold #00ffff")
        table.add_column("Service", style="bold #ffff00")
        table.add_column("Risk", style="bold white")
        table.add_column("Count", style="bold #00ff41", justify="right")

        for i, (port_str, count) in enumerate(ports_data, 1): 
            try:
                port = int(port_str)
                style, service = self._shodan_utils.get_port_style_and_service(port)
                
                if "red" in style:
                    risk = "[blink bold red]HIGH[/]"
                elif "yellow" in style:
                    risk = "[bold yellow]MEDIUM[/]"
                else:
                    risk = "[bold #00ff41]LOW[/]"
                
                table.add_row(
                    str(i),
                    f"[{style}]{port}[/]",
                    service,
                    risk,
                    f"[bold white]{count}[/]"
                )
            except ValueError:
                continue

        return table

    def _create_crypto_analysis(self, data: Dict) -> Table:
        table = Table(
            title="🔐 [bold #ffff00]Crypto Weaknesses[/]",
            box=box.SQUARE,
            border_style="bold #ffff00",
            style="bold #00ff41 on #0a0a0a"
        )
        table.add_column("Protocol", style="bold #00ffff")
        table.add_column("Count", style="bold #ff5555", justify="right")
        table.add_column("Risk", style="bold white")

        crypto_data = [
            ("SSLv2", data.get("numSsl2", 0), "[bold red]CRITICAL[/]"),
            ("SSLv3", data.get("numSsl3", 0), "[bold red]HIGH[/]"),
            ("TLS1.0", data.get("numTls1", 0), "[bold yellow]MEDIUM[/]"),
            ("TLS1.1", data.get("numTls11", 0), "[bold yellow]MEDIUM[/]"),
            ("TLS1.2", data.get("numTls12", 0), "[bold #00ff41]LOW[/]")
        ]

        for proto, count, risk in crypto_data:
            table.add_row(proto, str(count), risk)

        return table

    def display_reports(self, data: Dict):
        if not data:
            return

        threat_table = self._create_threat_overview(data)
        port_table = self._create_port_analysis(data.get("ports", []))
        crypto_table = self._create_crypto_analysis(data)

        critical_count = sum([
            data.get("numBluekeep", 0),
            data.get("numEternalblue", 0),
            data.get("numCompromisedDB", 0)
        ])
        
        threat_level = (
            "[blink bold red on #330000] CRITICAL THREATS [/]" if critical_count > 0 else
            "[bold yellow on #333300] WARNING [/]" if data.get("numSsl3", 0) > 100 else
            "[bold #00ff41 on #003300] SECURE [/]"
        )

        self.console.print(Panel(
            threat_table,
            title="[bold #ff5555] ShodanX EXPOSURE INTELLIGENCE [/]",
            border_style="bold #ff00ff",
            padding=(1, 2)
        ))

        self.console.print(Panel(
            port_table,
            title="[bold #00ffff] PORT VULNERABILITY ANALYSIS [/]",
            border_style="bold #00ffff")
        )

        self.console.print(Panel(
            crypto_table,
            title="[bold #ffff00] CRYPTOGRAPHIC WEAKNESSES ANALYSIS [/]",
            border_style="bold #ffff00")
        )

        self.console.print(Panel.fit(
            threat_level,
            title="[bold white] SYSTEM THREAT LEVEL [/]",
            border_style="bold red" if critical_count > 0 else "bold yellow"
        ))



import httpx
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from typing import List, Dict, Any
from shodanx.modules.logger.logger import Logger
from rich import box
from rich.text import Text
from rich.columns import Columns
from rich.align import Align

class ShodanxEntityDB:
    def __init__(self, session: httpx.AsyncClient):
        self._shodan_url = "https://entitydb.shodan.io/api"
        self.session = session
        self.console = Console()
        self.logger = Logger()

    async def list_all_entities(self) -> List[Dict[str, Any]]:
        url = f"{self._shodan_url}/entities/"
        try:
            response: httpx.Response = await self.session.get(url, timeout=30, follow_redirects=True)
            data = response.json()
            entities = data.get("entities", [])
            self.display_entities_table(entities=entities)
            return entities
        except Exception as e:
            self.logger.warn(f"Error occurred in the listing entity DB due to: {e}")
            return []
        
    def display_entities_table(self, entities: List[Dict[str, Any]]) -> None:
        table = Table(
            title="[bold white]ShodanX Entity DB - Companies List[/bold white]",
            box=box.DOUBLE_EDGE,
            border_style="bright_blue",
            header_style="bold bright_green",
            show_lines=True,
            expand=True
        )
    
        table.add_column("ID", style="bold green", justify="right", width=8)
        table.add_column("CIK", style="bold cyan", justify="right", width=12)
        table.add_column("Tickers", style="bold yellow", width=15)
        table.add_column("Entity Name", style="bold magenta", overflow="fold")

        for idx, ent in enumerate(entities):
            id = str(ent.get("id", "N/A"))
            cik = str(ent.get("cik", "N/A"))
            tickers = ", ".join(ent.get("tickers", [])) or "[dim]N/A[/dim]"
            name = ent.get("entity_name", "[dim]N/A[/dim]")
        
            row_style = "bold bright_white" if idx % 2 == 0 else "bold bright_cyan"
        
            if "N/A" not in cik:
                cik = f"[bold bright_green]{cik}[/bold bright_green]"
            if "N/A" not in tickers:
                tickers = f"[bold bright_yellow]{tickers}[/bold bright_yellow]"
        
            table.add_row(
                f"[green]{id}[/green]",
                cik,
                tickers,
                f"[bright_magenta]{name}[/bright_magenta]",
                style=row_style
            )

        panel = Panel.fit(
            table,
            title="[bold yellow]Corporate Entities Intelligence[/bold yellow]",
            subtitle="[dim]Data sourced from Shodan Entity Database[/dim]",
            border_style="bright_white",
            padding=(1, 2),
            style="on black"
        )
    
        self.console.print(panel)
        
    async def list_entity_by_id(self, id: int) -> dict:
        try:
            url = f"{self._shodan_url}/entities/{id}"
            response: httpx.Response = await self.session.request("GET", url, timeout=30, follow_redirects=True)
            if response.status_code != 200:
                return {}
            data = response.json()
            entity = data["entity"]
            self.display_entity_table(entity)
            return entity
        except Exception as e:
            self.logger.warn(f"Error occurred in the id entity db search module due to: {e}")
            return {}
            
    def display_entity_table(self, entity: Dict[str, Any]) -> None:
        header = Panel.fit(
            Text.from_markup(
            f"[bold bright_cyan]{entity.get('entity_name', 'N/A')}[/]\n"
            f"[bold yellow]{entity.get('entity_type', 'N/A')}[/]"
            ),
            border_style="bright_blue"
        )
        self.console.print(header)
    
        info_panel1 = Panel(
            Text.from_markup(
            f"🏛 [bold green]CIK:[/] [bright_white]{entity.get('cik', 'N/A')}[/]\n"
            f"📈 [bold green]Tickers:[/] [bright_yellow]{', '.join(entity.get('tickers', ['N/A']))}[/]\n"
            f"🏷 [bold green]SIC:[/] [white]{entity.get('sic', 'N/A')}[/] ([italic]{entity.get('sic_description', 'N/A')}[/])"
            ),
            title="[bold]Financial ID[/]",
            border_style="bright_green"
        )
    
        info_panel2 = Panel(
            Text.from_markup(
            f"🌐 [bold green]Website:[/] [bright_blue underline]{entity.get('hostname', 'N/A')}[/]\n"
            f"📞 [bold green]Phone:[/] [white]{entity.get('phone', 'N/A')}[/]\n"
            f"📅 [bold green]FY End:[/] [white]{entity.get('fiscal_year_end', 'N/A')}[/]"
            ),
            title="[bold]Contact Info[/]",
            border_style="bright_blue"
        )
    
        self.console.print(Columns([info_panel1, info_panel2]))
    
        address_panel = Panel(
            Text.from_markup(
            f"🏢 [bold green]Address:[/]\n[white]{entity.get('business_address', 'N/A')}[/]"
            ),
            border_style="bright_magenta"
        )
        self.console.print(address_panel)
    
        asn_table = Table(
            title="[bold yellow]NETWORK FOOTPRINT[/bold yellow]",
            box=box.DOUBLE_EDGE,
            border_style="bright_yellow",
            header_style="bold bright_cyan",
            show_lines=True,
            expand=True
        )
    
        asn_table.add_column("ASN", style="bold blue", justify="right")
        asn_table.add_column("CIDRs", style="bold white", justify="center")
        asn_table.add_column("Total IPs", style="bold magenta", justify="right")
        asn_table.add_column("Threat Profile", style="bold red", justify="center")
    
        ip_count = 0
        for asn_data in entity.get("asns", []):
            asn = str(asn_data.get("asn", "N/A"))
            prefixes = asn_data.get("route_views", [])
            cidr_count = len(prefixes)
        
            asn_ip_count = 0
            for p in prefixes:
                try:
                    asn_ip_count += int(p.get("number_ips", 0))
                except (ValueError, TypeError):
                    pass
        
            ip_count += asn_ip_count
        
            threat_level = "🟢 Low" if asn_ip_count < 1000 else \
                      "🟡 Medium" if asn_ip_count < 10000 else \
                      "🔴 High" if asn_ip_count < 50000 else \
                      "💀 Critical"
        
            asn_table.add_row(
            f"[bold bright_blue]{asn}[/]",
            f"[bold]{cidr_count:,}[/]",
            f"[bold bright_magenta]{asn_ip_count:,}[/]",
            f"[bold]{threat_level}[/]"
            )

        all_prefixes = [p.get('prefix', 'N/A') for asn in entity.get('asns', []) for p in asn.get('route_views', [])]
        if all_prefixes:
            largest_cidr = max(all_prefixes, key=lambda x: len(x) if x != 'N/A' else 0)
        else:
            largest_cidr = "N/A (No prefixes available)"

        network_panel = Panel(
            Text.from_markup(
            f"[bold]🌍 Total Network Exposure:[/] [bright_red]{ip_count:,}[/] IP addresses across [bright_yellow]{len(entity.get('asns', []))}[/] ASNs\n"
            f"[bold]🔥 Largest CIDR Block:[/] [bright_white]{largest_cidr}[/]"
            ),
            title="[bold]NETWORK THREAT ASSESSMENT[/bold]",
            border_style="yellow",
            style="on black"
            )
    
        self.console.print("\n")
        self.console.print(network_panel)
        self.console.print("\n")
        self.console.print(asn_table)
    
    # just a suggestion :)
        if ip_count > 10000:
            security_panel = Panel(
            Text.from_markup(
                "[bold bright_red]⚠️ HIGH RISK TARGET ⚠️[/]\n"
                "This organization has significant internet exposure\n"
                "Recommended recon techniques:\n"
                "• [bold]Masscan[/] for rapid port scanning\n"
                "• [bold]Shodan[/] queries for exposed services\n"
                "• [bold]ASN[/] mapping for network topology"
            ),
            border_style="bold red",
            style="on bright_black"
            )
            self.console.print("\n")
            self.console.print(security_panel)